package edu.umich.visualsoar.misc;
import edu.umich.visualsoar.operatorwindow.OperatorNode;



/**
 * Just used to pass some things around
 * @author Brad Jones
 * @version 0.5a 5 Aug 1999
 */
public class FeedbackListObject  {
///////////////////////////////////////////////////////////////////
// Data Members
///////////////////////////////////////////////////////////////////
	private OperatorNode node;
	private int lineNumber;
	private String message;
	private boolean msgEnough = false;
	private boolean d_isError = false;
  private boolean d_isGenerated = false;
	
///////////////////////////////////////////////////////////////////
// Constructors
///////////////////////////////////////////////////////////////////
	// deny default construction
	private FeedbackListObject() {}
	
	/**
	 * Create the List Object with the given parameters
	 */
	public FeedbackListObject(OperatorNode in_node,int in_ln, String msg) {
		node = in_node;
		lineNumber = in_ln;
		message = msg;
	}
	
	public FeedbackListObject(OperatorNode in_node,int in_ln, String msg, boolean _msgEnough) {
		this(in_node,in_ln,msg);
		msgEnough = _msgEnough;
	}
	
	public FeedbackListObject(OperatorNode in_node,int in_ln,String msg,boolean _msgEnough,boolean isError) {
		this(in_node,in_ln,msg,_msgEnough);
		d_isError = isError;
	}

  public FeedbackListObject(OperatorNode in_node,int in_ln,String msg,boolean _msgEnough,boolean isError, boolean isGenerated) {
		this(in_node,in_ln,msg,_msgEnough);
		d_isGenerated = isGenerated;
	}
	
///////////////////////////////////////////////////////////////////
// Methods
///////////////////////////////////////////////////////////////////
	public boolean isError() {
		return d_isError;
	}

  public boolean isGenerated() {
    return d_isGenerated;
  }
	
	/**
	 * returns the filename of the file for which the node of this
	 * list object is associated
	 * @return a string which is the file name of the file of which this object is associated
	 */
	public String getFileName() {
		return node.getFileName();
	}
	
	/**
	 * returns the node for which this object is associated
	 * @return a reference to the node
	 */
	public OperatorNode getNode() {
		return node;
	}
	
	/**
	 * returns the line number for which this object is associated
	 * @return an int that is positive
	 */
	public int getLine() {
		return lineNumber;
	}
	
	/**
	 * returns the message for this object
	 * @return a string that is additional information for the user
	 */
	public String getMessage() {
		return message;
	}
	
	/**
	 * lets you set a new message
	 * @param s a string that is the new message
	 */
	public void setMessage(String s) {
		message = s;
	}	
	
	/**
	 * returns a string to represent this object, it is a combination of
	 * the file, line number and the message
	 * @return a string that represents this object
	 */
	public String toString() {
		if (msgEnough) 
			return message;
		else
			return node.getUniqueName() + "(" + lineNumber + "): " + message;
	} 
}
